@@ -25,6 +25,7 @@
   #:use-module (ice-9 match)
   #:use-module (ice-9 format)
   #:use-module (ice-9 rdelim)   ;; Line-based I/O.
+  #:use-module ((ice-9 textual-ports) #:select (put-char))
   #:use-module ((ice-9 threads) #:select (all-threads))
   #:use-module (srfi srfi-1)    ;; List library.
   #:use-module (srfi srfi-26)
@@ -307,6 +308,8 @@ configuration file '~a': ~s")
            (match (accept sock (logior SOCK_NONBLOCK SOCK_CLOEXEC))
              ((command-source . client-address)
               (setvbuf command-source 'block 1024)
+              (set-port-encoding! command-source "UTF-8")
+              (set-port-conversion-strategy! command-source 'error)
               (spawn-fiber
                (lambda ()
                  (process-connection command-source))))
@@ -408,6 +411,7 @@ fork in the child process."
   (initialize-cli)
 
   (let ((config-file #f)
+        (syslog-requested? #f)
 	(socket-file default-socket-file)
         (pid-file    #f)
         (secure      #t)
@@ -469,14 +473,25 @@ fork in the child process."
 		    #:action (lambda (file)
 			       (set! socket-file
 				     (and (not (string=? file "-"))
-					  file)))))
+					  file))))
+                  (option
+                   #:long-name "syslog"
+                   #:takes-argument? #t #:argument-is-optional? #t
+                   #:argument-name (l10n "FILE")
+                   #:description
+                   (l10n "log to the system log (syslog) at FILE or /dev/log")
+                   #:action (lambda (file)
+                              (when file
+                                (system-log-file file))
+                              (set! syslog-requested? #t))))
     ;; We do this early so that we can abort early if necessary.
     (and socket-file
          (verify-dir (dirname socket-file) #:secure? secure))
 
     (define syslog?
       ;; Is shepherd logging to /dev/log?
-      (and (not logfile) (= 1 (getpid))))
+      (or syslog-requested?
+          (and (not logfile) (= 1 (getpid)))))
 
     ;; By default, enable detailed deprecation warnings.
     (unless (getenv "GUILE_WARN_DEPRECATED")
@@ -500,10 +515,15 @@ fork in the child process."
                          ;; stdout.  Redirect stdout to the bitbucket so we
                          ;; don't log twice.
                          (%make-void-port "w")
-                         (current-output-port)))))
+                         (current-output-port))))
+
+                   ;; In Guile 3.0.10, calling 'environ' from the top-level
+                   ;; triggers a warning so do it from here.
+                   (default-environment-variables (environ)))
 
       (parameterize ((current-output-port (%current-service-output-port)))
         (set-port-encoding! (log-output-port) "UTF-8")
+        (set-port-encoding! (%current-service-output-port) "UTF-8")
 
         ;; Log provenance info.
         (format #t "~a ~a (Guile ~a, ~a)~%"
@@ -559,6 +579,10 @@ fork in the child process."
                ;; not introduce a continuation barrier.  Replace 'sleep' to
                ;; avoid blocking in user code such as 'start' methods.
                (replace-core-bindings!
+                (newline (lambda* (#:optional (port (current-output-port)))
+                           ;; As of Guile 3.0.10, 'newline' is written in C
+                           ;; and thus a continuation barrier.  Replace it.
+                           (put-char port #\newline)))
                 (system* (lambda command
                            (spawn-command command)))
                 (system spawn-shell-command)
