@@ -1,5 +1,5 @@
 ;; args.scm -- Command line argument handling.
-;; Copyright (C) 2013, 2016, 2018, 2023 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013, 2016, 2018, 2023-2024 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Jährling <wolfgang@pro-linux.de>
 ;;
 ;; This file is part of the GNU Shepherd.
@@ -87,8 +87,13 @@ characteristics."
       (set! col (+ col (string-length text)))
       (display text))
     (define (fill-to target-col)
-      (while (< col target-col)
-	     (output (string #\space))))
+      (if (>= col target-col)
+          (begin
+            (set! col 0)
+            (newline)
+            (fill-to target-col))
+          (while (< col target-col)
+	    (output (string #\space)))))
 
     (fill-to 2)
     (when (option-short-name opt)
@@ -100,6 +105,9 @@ characteristics."
     (output (option-description opt)))
   (newline))
 
+(define general-help-url
+  "https://www.gnu.org/gethelp/")
+
 ;; Interpret command line arguments ARGS according to OPTIONS, passing
 ;; non-option arguments to DEFAULT.  Uses `program-name' and
 ;; `bug-address', which must be defined elsewhere.
@@ -216,10 +224,11 @@ optional to the corresponding short options.
 
 Report bugs to: ~a .
 ~a general home page: <~a>
-General help using GNU software: <http://www.gnu.org/gethelp/>~%")
+General help using GNU software: <~a>~%")
                                     bug-address
                                     package-name
-                                    package-url)
+                                    package-url
+                                    general-help-url)
 			    (quit)))
 	       options))
   (let next-arg ((ptr args))
@@ -255,12 +264,12 @@ General help using GNU software: <http://www.gnu.org/gethelp/>~%")
 		 (set! target-option (find-long-option name))
 		 (and (boolean? target-option)
 		      (begin
-			(local-output
-			 (if target-option
-			     (l10n "Option `--~a' is ambiguous.")
-			   (l10n "Unknown option: `--~a'."))
-			 name)
-			(local-output (l10n "Try `--help'."))
+                        (if target-option
+                            (report-error (l10n "option '--~a' is ambiguous")
+                                          name)
+                            (report-error (l10n "unknown option '--~a'")
+                                          name))
+			(report-hint (l10n "Try '--help'."))
 			(quit 1)))
 		 (and (option-takes-argument? target-option)
 		      (not (optional-arg? target-option))
@@ -284,7 +293,7 @@ General help using GNU software: <http://www.gnu.org/gethelp/>~%")
 		    (param #f))
 	       (if (not target-option)
 		   (begin
-		     (local-output (l10n "Unknown option: `-~a'.")
+		     (report-error (l10n "unknown option '-~a'")
                                    opt-char)
 		     (quit 1))
 		 (if (option-takes-argument? target-option)
