@@ -1,5 +1,5 @@
 ;; comm.scm -- Communication between processes and general output.
-;; Copyright (C) 2013-2014, 2016, 2018-2019, 2022-2024 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013-2014, 2016, 2018-2019, 2022-2025 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Jährling <wolfgang@pro-linux.de>
 ;; Copyright (C) 2018 Danny Milosavljevic <dannym@scratchpost.org>
 ;;
@@ -22,7 +22,9 @@
   #:use-module (shepherd support)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-9)
+  #:autoload   (ice-9 control) (let/ec)
   #:use-module (ice-9 match)
+  #:autoload   (ice-9 suspendable-ports) (current-write-waiter)
   #:autoload   (ice-9 textual-ports) (put-string)
   #:export (open-connection
             open-server-socket
@@ -120,7 +122,10 @@ return the socket."
 wrong---premature end-of-file, invalid sexp, etc."
   (catch 'read-error
     (lambda ()
-      (match (read port)
+      (match (catch 'decoding-error
+               (lambda ()
+                 (read port))
+               (const #f))
         (('shepherd-command ('version version ...)
                             ('action action)
                             ('service service)
@@ -278,35 +283,39 @@ mechanism."
                   (call-with-syslog-port proc))
                 (apply throw args)))))
 
-      (or (and port (not (port-closed? port)) (call/syslog))
-          (let ((sock (socket AF_UNIX
-                              (logior SOCK_CLOEXEC SOCK_NONBLOCK SOCK_DGRAM)
-                              0)))
-            (catch 'system-error
-              (lambda ()
-                (connect sock AF_UNIX (system-log-file))
-                (setvbuf sock 'line)
-                (set! port sock)
-                (call/syslog))
-              (lambda args
-                (close-port sock)
-                (if (memv (system-error-errno args)
-                          (list ENOENT ECONNREFUSED))
-                    (catch 'system-error
-                      (lambda ()
-                        (call-with-output-file "/dev/kmsg"
-                          (lambda (port)
-                            (setvbuf port 'block)
-                            (proc (non-blocking-port port)))))
-                      (lambda args
-                        (if (memv (system-error-errno args)
-                                  (list ENOENT EACCES EPERM))
-                            (call-with-output-file "/dev/console"
-                              (lambda (port)
-                                (setvbuf port 'none)
-                                (proc (non-blocking-port port))))
-                            (apply throw args))))
-                    (apply throw args)))))))))
+      (define (try-open file)
+        (catch 'system-error
+          (lambda ()
+            (let* ((fdes (open-fdes file
+                                    (logior O_NONBLOCK O_CLOEXEC O_WRONLY)))
+                   (port (fdopen fdes "w")))
+              (setvbuf port 'line)
+              port))
+          (lambda args
+            (if (memv (system-error-errno args)
+                      (list ENOENT EACCES EPERM))
+                #f
+                (apply throw args)))))
+
+      (with-fluids ((%default-port-encoding "UTF-8"))
+        (or (and port (not (port-closed? port)) (call/syslog))
+            (let ((sock (socket AF_UNIX
+                                (logior SOCK_CLOEXEC SOCK_NONBLOCK SOCK_DGRAM)
+                                0)))
+              (catch 'system-error
+                (lambda ()
+                  (connect sock AF_UNIX (system-log-file))
+                  (setvbuf sock 'line)
+                  (set! port sock)
+                  (call/syslog))
+                (lambda args
+                  (close-port sock)
+                  (if (memv (system-error-errno args)
+                            (list ENOENT ECONNREFUSED))
+                      (or (and=> (try-open "/dev/kmsg") proc)
+                          (and=> (try-open "/dev/console") proc)
+                          (current-output-port))
+                      (apply throw args))))))))))
 
 ;; We provide our own output mechanism, because we have certain
 ;; special needs; most importantly, we want to send output to herd
@@ -335,19 +344,30 @@ mechanism."
         (if (not (string-index str #\newline))
             (set! buffer (cons str buffer))
             (let ((str (string-concatenate-reverse (cons str buffer))))
-              ;; Note: We want to render as many newlinew as present in STR,
-              ;; so neither 'string-split' nor 'string-tokenize' helps.
-              (let loop ((str str))
-                (let* ((index (string-index str #\newline))
-                       (line  (if index (string-take str (+ 1 index)) str)))
-                  (unless (string-null? str)
-                    ;; Make exactly one 'put-string' call per line to make
-                    ;; sure we don't create several entries for each line.
-                    (put-string (log-output-port) line)
-                    (when index
-                      (loop (string-drop str (+ index 1)))))))
+              (let/ec return
+                ;; Replace Fibers' default write waiter because this
+                ;; continuation cannot be suspended: as of Guile 3.0.9, soft
+                ;; ports are implemented in C and are a continuation barrier.
+                ;; Thus, if PORT is not ready for writing, simply bail out and
+                ;; keep the output for later.
+                ;; TODO: Rewrite using 'make-custom-port' in Guile >= 3.0.10.
+                (parameterize ((current-write-waiter
+                                (lambda (_)
+                                  (return #t))))
+                  ;; Note: We want to render as many newlines as present in
+                  ;; STR, so neither 'string-split' nor 'string-tokenize'
+                  ;; helps.
+                  (let loop ((str str))
+                    (let* ((index (string-index str #\newline))
+                           (line  (if index (string-take str (+ 1 index)) str)))
+                      (unless (string-null? str)
+                        ;; Make exactly one 'put-string' call per line to make
+                        ;; sure we don't create several entries for each line.
+                        (put-string (log-output-port) line)
+                        (when index
+                          (loop (string-drop str (+ index 1)))))))
 
-              (set! buffer '())))))
+                  (set! buffer '())))))))
 
     ;; Flush output.
     (lambda ()
