@@ -67,7 +67,7 @@ rdelim)."
     (and (< i len)
          (match (read-char port)
            ((? eof-object? eof)
-            eof)
+            (if (zero? i) eof i))
            ((or #\newline #\return)
             i)
            (chr
@@ -317,14 +317,12 @@ to @var{history-size} lines in memory."
                                  (process-id process))
                                 (value
                                  value)))))
-                (now (current-time)))
-           (if (integer? pid)
-               (simple-format (log-output-port) "~a[~a] "
-                              command pid)
-               (simple-format (log-output-port) "[~a] "
-                              command))
-           (put-string (log-output-port) line)
-           (newline (log-output-port))
+                (now (current-time))
+                ;; Note: 'simple-format' is not suspendable, hence 'put-string'.
+                (line* (if (integer? pid)
+                           (simple-format #f "~a[~a] ~a~%" command pid line)
+                           (simple-format #f "[~a] ~a~%" command line))))
+           (put-string (log-output-port) line*)
            (loop pid
                  (ring-buffer-insert (cons now line)
                                      messages)
