@@ -1,5 +1,5 @@
 ;; log-rotation.scm -- Rotating service log files.
-;; Copyright (C) 2024 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2024-2025 Ludovic Courtès <ludo@gnu.org>
 ;;
 ;; This file is part of the GNU Shepherd.
 ;;
@@ -119,7 +119,7 @@ called with @var{file}."
   ;; leaks and to reduce the attack surface.
   (match method
     ('gzip (system* %gzip-program "-9" file))
-    ('zstd (system* %zstd-program "-9" file))
+    ('zstd (system* %zstd-program "-9" "--rm" file))
     ('none #f)
     ((? procedure? proc) (proc file))
     (_  #f)))
@@ -139,19 +139,24 @@ called with @var{file}."
      (local-output (l10n "Log file '~a' is inaccessible; not rotating.")
                    file))
     (stat
-     (if (> (stat:size stat) rotation-size-threshold)
-         (begin
-           ;; First rotate past logs, to free "FILE.1".
-           (rotate-past-logs file rotate)
-           ;; Then rename FILE to "FILE.1".
-           (let* ((next (string-append file ".1"))
-                  (rotated? (rotate-current file next)))
-             (when rotated?
-               (compress-file compression next)
-               (local-output (l10n "Rotated '~a'.") file))))
-         (local-output
-          (l10n "Not rotating '~a', which is below the ~a B threshold.")
-          file rotation-size-threshold)))))
+     (cond ((not (eq? 'regular (stat:type stat)))
+            ;; Do not rotate /dev/tty12, directories, or anything like that.
+            (local-output
+             (l10n "Not rotating '~a', which is not a regular file.")
+             file))
+           ((> (stat:size stat) rotation-size-threshold)
+            ;; First rotate past logs, to free "FILE.1".
+            (rotate-past-logs file rotate)
+            ;; Then rename FILE to "FILE.1".
+            (let* ((next (string-append file ".1"))
+                   (rotated? (rotate-current file next)))
+              (when rotated?
+                (compress-file compression next)
+                (local-output (l10n "Rotated '~a'.") file))))
+           (else
+            (local-output
+             (l10n "Not rotating '~a', which is below the ~a B threshold.")
+             file rotation-size-threshold))))))
 
 (define* (rotate-logs logger #:optional (rotate rename-file)
                       #:key
