@@ -1,5 +1,5 @@
 ;; timer.scm -- Timer service.
-;; Copyright (C) 2024 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2024-2025 Ludovic Courtès <ludo@gnu.org>
 ;;
 ;; This file is part of the GNU Shepherd.
 ;;
@@ -43,6 +43,7 @@
             cron-string->calendar-event
 
             next-calendar-event
+            seconds-to-wait
 
             command
             command?
@@ -127,6 +128,8 @@ names to indexes, and @var{index->symbol} that does the opposite."
                  (unless (and (>= value min) (<= value max))
                    (raise (condition
                            (&message
+                            ;; TRANSLATORS: 'calendar-event' is the name of a
+                            ;; procedure and it must be kept unchanged.
                             (message (format #f (l10n "calendar-event: ~a: \
 ~a: value out of range (~a-~a)")
                                              'lst value min max)))))))
@@ -171,6 +174,7 @@ invalid day of week")
 (define-date-setter set-date-day date-day)
 (define-date-setter set-date-month date-month)
 (define-date-setter set-date-year date-year)
+(define-date-setter set-date-zone-offset date-zone-offset)
 
 (define (increment-year date)
   (set-date-year date (+ 1 (date-year date))))
@@ -320,9 +324,19 @@ Sunday), return the corresponding list of days in @var{month} of @var{year}."
                   days))
         (reverse days))))
 
+(define (adjust-date-timezone-offset date)
+  "Reset the timezone offset of the given date such that it matches the
+local timezone offset at that time, taking DST into account."
+  ;; This trick relies on the fact that the second argument of
+  ;; 'time-utc->date' defaults to the local timezone offset for the given
+  ;; time.
+  (let ((offset (date-zone-offset (time-utc->date (date->time-utc date)))))
+    (set-date-zone-offset date offset)))
+
 (define (next-calendar-event event date)
   "Return the date following @var{date} that matches @var{event}, a calendar
-event record."
+event record.  The returned date has its @code{zone-offset} field adjusted to
+represent the local time on that date, taking DST into account."
   (define (month date)
     (if (memv (date-month date) (calendar-event-months event))
         date
@@ -357,14 +371,13 @@ event record."
     ;; Clear nanoseconds and jump to the next second.
     (increment-second (set-date-nanosecond date 0)))
 
-  ;; TODO: Adjust timezone.
-  (month (day (hour (minute (second (nanosecond date)))))))
+  (adjust-date-timezone-offset
+   (month (day (hour (minute (second (nanosecond date))))))))
 
-(define (seconds-to-wait event)
-  "Return the number of seconds to wait before the next occurrence of
-@var{event} (the result is an inexact number, always greater than zero)."
-  (let* ((now (current-time time-utc))
-         (then (next-calendar-event event (time-utc->date now)))
+(define* (seconds-to-wait event #:optional (now (current-time time-utc)))
+  "Return the number of seconds to wait from @var{now} until the next occurrence
+of @var{event} (the result is an inexact number, always greater than zero)."
+  (let* ((then (next-calendar-event event (time-utc->date now)))
          (diff (time-difference (date->time-utc then) now)))
     (+ (time-second diff)
        (/ (time-nanosecond diff) 1e9))))
@@ -410,6 +423,8 @@ five minutes after midnight, every day;
                 (in-range? min) (in-range? max)
                 (iota (floor-quotient (+ 1 (- max min)) step)
                       min step)))
+          (("*")
+           (iota (ceiling-quotient count step) min step))
           (((= string->number n))
            (and (in-range? n) (list n)))
           (_ #f))))
@@ -447,7 +462,7 @@ five minutes after midnight, every day;
                                      (fail 'id))))
                               ((_ () args)
                                (apply calendar-event args)))))
-       (parse ((minutes 60 0) (hours 60 0)
+       (parse ((minutes 60 0) (hours 24 0)
                (days-of-month 31 1) (months 12 1) (days-of-week 7 0))
               '())))
     (_
@@ -673,14 +688,18 @@ seconds, the timer's process is forcefully terminated with signal
                   (put-message reply #t)))
              (('process-terminated pid status)
               ;; Process PID completed.
-              (let ((start-time (assoc-ref processes pid))
-                    (end-time ((@ (guile) current-time)))
-                    (remaining (alist-delete pid processes)))
+              (let* ((start-time (assoc-ref processes pid))
+                     (end-time ((@ (guile) current-time)))
+                     (remaining (alist-delete pid processes))
+                     (duration (- end-time start-time)))
                 (local-output
                  (l10n "Process ~a of timer '~a' terminated with status ~a \
-after ~a seconds.")
+after ~a second."
+                       "Process ~a of timer '~a' terminated with status ~a \
+after ~a seconds."
+                       duration)
                  pid name status
-                 (- end-time start-time))
+                 duration)
                 (if (and termination (null? remaining))
                     (put-message termination #t) ;done
                     (begin
@@ -723,7 +742,10 @@ after ~a seconds.")
                                                        'timeout)
                                     ('timeout
                                      (local-output (l10n "Terminating \
-process ~a of timer '~a' after maximum duration of ~a seconds.")
+process ~a of timer '~a' after maximum duration of ~a second."
+                                                         "Terminating \
+process ~a of timer '~a' after maximum duration of ~a seconds."
+                                                         max-duration)
                                                    pid name max-duration)
                                      (terminate-process pid termination-signal)
                                      (get-message status))
@@ -867,6 +889,10 @@ strings) to run on the first occurrence of that event.  Raise an error if
     (match spec
       (()
        (unless event
+         ;; TRANSLATORS: In the 'at TIME' string, only 'TIME' may be
+         ;; translated.  It refers to the fact that the user ran 'herd
+         ;; schedule timer COMMAND' instead of 'herd schedule timer at 12:00
+         ;; COMMAND', say, where '12:00' is a possible value for TIME.
          (invalid-argument (l10n "Timer spec lacks 'at TIME'.")))
        (unless (pair? arguments)
          (invalid-argument (l10n "Timer spec lacks a command to run.")))
@@ -874,9 +900,7 @@ strings) to run on the first occurrence of that event.  Raise an error if
       (("at" time-spec rest ...)
        (loop rest (string->calendar-event time-spec) arguments))
       ((lst ...)
-       (if event
-           (loop '() event lst)
-           (invalid-argument (l10n "Timer spec lacks 'at TIME'.")))))))
+       (loop '() event lst)))))
 
 (define* (schedule-timer spec
                          #:key
@@ -951,6 +975,7 @@ schedule command execution similar to the venerable @command{at} command."
 follows '--' to run at 10AM.")))
 
   (service provision
+           #:requirement requirement
            #:start (const #t)
            #:stop (const #f)
            #:actions (list schedule-action)
