@@ -437,16 +437,16 @@ TARGET should be a string representing a filepath + name."
         (l10n "\
 ;; Services known to shepherd:
 ;; Add new services (defined using 'service') to shepherd here by
-;; providing them as arguments to 'register-services'.\n")
-        "(register-services)\n\n"
+;; providing them as a list to 'register-services'.\n")
+        "(register-services (list))\n\n"
         (l10n "\
-;; Send shepherd into the background\n")
+;; Send shepherd into the background.\n")
         "(perform-service-action root-service 'daemonize)\n\n"
         (l10n "\
 ;; Services to start when shepherd starts:
 ;; Add the name of each service that should be started to the list
 ;; below passed to 'for-each'.\n")
-        "(for-each start '())\n")))))
+        "(start-in-the-background '())\n")))))
 
 ;; Logging.
 (define (user-default-log-file)
