@@ -1,5 +1,5 @@
 # GNU Shepherd --- Ensure file descriptors are not leaked to children.
-# Copyright © 2022-2024 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2022-2025 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -82,9 +82,10 @@ main (int argc, char *argv[])
     }
   closedir (dir);
 
-  log = fopen ("$fd_count", "w");
+  log = fopen ("$fd_count.tmp", "w");
   fprintf (log, "%zi\n", count);
   fclose (log);
+  rename ("$fd_count.tmp", "$fd_count");
 
   return EXIT_SUCCESS;
 }
@@ -172,7 +173,7 @@ connect_to_server ()
       (define address (make-socket-address AF_INET INADDR_LOOPBACK $1))
       (define sock (socket (sockaddr:fam address) SOCK_STREAM 0))
       (connect sock address)"
-    while ! test -f "$fd_count" ; do sleep 0.3 ; done
+    until test -s "$fd_count"; do sleep 0.3; done
 }
 
 for i in $(seq 1 3)
