@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test transient services.
-# Copyright © 2022-2024 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2022-2025 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -131,7 +131,10 @@ guile -c "
   (connect sock address)
   (sleep 300)" &
 client_pid=$!
-$herd status test-inetd-1
+
+# Wait until the inetd service shows up.
+until $herd status test-inetd-1; do sleep 0.3; done
+
 $herd status test-inetd-1 | grep transient
 $herd status test-inetd-1 | grep "Command:.*$SHELL.*echo hello"
 $herd stop test-inetd-1
