@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test the logging capabilities of 'make-forkexec-constructor'.
-# Copyright © 2022, 2023 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2022-2025 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -56,6 +56,16 @@ cat > "$conf"<<EOF
 	 #:stop  (make-kill-destructor)
 	 #:respawn? #f)
 
+       (service
+         ;; Service with built-in logging, not emitting a newline in its log.
+         '(test-logging-without-newline)
+         #:start (make-forkexec-constructor
+                  '("$SHELL" "-c" "echo \$first; echo -n \$second")
+                  #:environment-variables
+                  '("first=aaaaa" "second=bbbbb"))
+         #:stop (make-kill-destructor)
+         #:respawn? #f)
+
        (service
 	 ;; Service with built-in logging.
 	 '(test-file-logging)
@@ -86,6 +96,14 @@ do
     grep -E '^2[0-9]{3}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2} '"$message" "$service_log"
 done
 
+$herd status test-file-logging | grep "Log file: $PWD/$service_log"
+
+$herd status test-file-logging | \
+    grep -E '^  2[0-9]{3}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2} STARTING'
+$herd status test-file-logging | \
+    grep -E '^  2[0-9]{3}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2} STARTED'
+
+
 # Return the file descriptor corresponding to the given file.
 find_file_descriptor ()
 {
@@ -117,7 +135,22 @@ rm -f "$service_pid"
 $herd start test-builtin-logging
 for message in "STARTING" "STARTED" "café" "latin1 garbage: .* alors"
 do
-    grep -E '^2[0-9]{3}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2} .*'"$message" "$log"
+    grep -E '^2[0-9]{3}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2} \['"$(basename $SHELL)"'\] '"$message" "$log"
 done
 
+$herd status test-builtin-logging | grep -v "Log file"
+$herd status test-builtin-logging | \
+    grep -E '^  2[0-9]{3}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2} STARTING'
+$herd status test-builtin-logging | \
+    grep -E '^  2[0-9]{3}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2} STARTED'
+
+# Check what happens when logging without a newline.
+$herd start test-logging-without-newline
+until $herd status test-logging-without-newline | grep stopped; do sleep 0.3; done
+
+$herd status root | grep aaaaa
+$herd status root | grep bbbbb
+grep aaaaa "$log"
+grep bbbbb "$log"
+
 $herd stop root
