@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test monitoring service.
-# Copyright © 2023 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2023-2024 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -23,12 +23,13 @@ socket="t-socket-$$"
 conf="t-conf-$$"
 log="t-log-$$"
 pid="t-pid-$$"
+witness="t-repl-witness-$$"
 repl_socket="$PWD/repl-socket-$$"
 
 herd="herd -s $socket"
 
 trap "cat $log || true;
-      rm -f $socket $repl_socket $conf $log;
+      rm -f $socket $repl_socket $conf $log $witness;
       test -f $pid && kill \`cat $pid\` || true; rm -f $pid" EXIT
 
 cat > "$conf" <<EOF
@@ -120,6 +121,31 @@ grep "heap:" "$log"
 
 $herd log monitoring | grep "heap:"
 
+# What if we evaluate code that raises an exception?  Does the REPL remain
+# functional?  It used to enter an infinite loop while spawning the debugger,
+# due to non-suspendable continuations:
+# <https://lists.gnu.org/archive/html/guix-devel/2024-01/msg00064.html>.
+rm -f "$witness"
+guile -c '
+(alarm 10)
+(let ((sock (socket AF_UNIX SOCK_STREAM 0)))
+  (connect sock PF_UNIX "'$repl_socket'")
+  (format #t "connected!~%> ")
+  (display "this-is-unbound-and-thus-throws" sock)
+  (newline sock)
+  (display (object->string (quote (open-output-file "'$witness'"))) sock)
+  (newline sock)
+  (display ",q\n" sock)
+
+  (let loop ()
+    (define chr (read-char sock))
+    (unless (eof-object? chr)
+      (display chr)
+      (when (eq? chr #\newline)
+	(display "> "))
+      (loop))))'
+test -f "$witness"
+
 $herd stop repl
 $herd status repl | grep "stopped"
 
